from fire import Fire

from torchyolo.automodel import AutoDetectionModel


class YoloHub:
    def __init__(self, config_path: str = None, model_type: str = "yolov5", model_path: str = None):
        self.config_path = config_path
        self.model_type = model_type
        self.model_path = model_path
        self.load_model()

    def load_model(self):
        self.model = AutoDetectionModel.from_pretrained(
            config_path=self.config_path,
            model_type=self.model_type,
            model_path=self.model_path,
        )

        return self.model

    def view_model_architecture(self, file_format: str = "pdf"):
        try:
            from torchview import draw_graph
        except:
            raise ImportError("Please install torchview: pip install torchview")

        if self.model_type == "yolov5" or self.model_type == "yolov7":
            model_arch = self.model.model

        elif self.model_type == "yolov8" or self.model_type == "yolox":
            model_arch = self.model.model.model

        elif self.model_type == "yolov6":
            model_arch = self.model.model.model.model

        model_graph = draw_graph(
            model_arch,
            input_size=(1, 3, 384, 640),
            expand_nested=True,
            depth=3,
        )

        model_graph.visual_graph.render(format=file_format)
        return model_graph

    def predict(
        self,
        source: str = None,
        tracker_type: str = None,
        tracker_weight_path: str = None,
        tracker_config_path: str = None,
    ):
        pred = self.model.predict(
            source=source,
            tracker_type=tracker_type,
            tracker_weight_path=tracker_weight_path,
            tracker_config_path=tracker_config_path,
        )
        return pred


def main(
    config_path: str = "torchyolo/configs/default_config.yaml",
    model_type: str = "yolov5",
    model_path: str = "yolov5s.pt",
    source: str = "../videos/test.mp4",
    tracker_type: str = None,
    tracker_weight_path: str = None,
    tracker_config_path: str = None,
):
    model = YoloHub(
        config_path=config_path,
        model_type=model_type,
        model_path=model_path,
    )
    result = model.predict(
        source=source,
        tracker_type=tracker_type,
        tracker_weight_path=tracker_weight_path,
        tracker_config_path=tracker_config_path,
    )
    return result


if __name__ == "__main__":
    Fire(main)



from fire import Fire

from torchyolo import __version__ as torchyolo_version
from torchyolo.predict import main

torchyolo_app = {
    "version": torchyolo_version,
    "predict": main,
}


def app():
    Fire(torchyolo_app)


if __name__ == "__main__":
    app()



MODEL_TYPE_TO_MODEL_CLASS_NAME = {
    "yolov5": "Yolov5DetectionModel",
    "yolov7": "Yolov7DetectionModel",
    "yolov6": "Yolov6DetectionModel",
    "yolov8": "Yolov8DetectionModel",
}


class AutoDetectionModel:
    def from_pretrained(config_path: str, model_type: str = "yolov5", model_path: str = "yolov5s.pt"):

        model_class_name = MODEL_TYPE_TO_MODEL_CLASS_NAME[model_type]
        DetectionModel = getattr(
            __import__(f"torchyolo.modelhub.{model_type}", fromlist=[model_class_name]), model_class_name
        )

        return DetectionModel(
            config_path=config_path,
            model_path=model_path,
        )



from pathlib import Path
from typing import Optional

from torchyolo.utils.config_utils import get_config

DEFAULT_BYTETRACK_CONFIG_PATH = "trackerhub/configs/byte_track.yaml"
DEFAULT_OCSORT_CONFIG_PATH = "trackerhub/configs/oc_sort.yaml"
DEFAULT_NORFAIR_CONFIG_PATH = "trackerhub/configs/norfair_track.yaml"
DEFAULT_STRONGSORT_CONFIG_PATH = "trackerhub/configs/strong_sort.yaml"
DEFAULT_SORT_CONFIG_PATH = "trackerhub/configs/sort_track.yaml"


def create_tracker(
    tracker_type,
    tracker_config_path,
    tracker_weight_path: Optional[str] = None,
    device: Optional[str] = "cpu",
    half: Optional[bool] = False,
    conf_th: Optional[str] = 0.05,
    iou_th: Optional[str] = 0.05,
) -> object:
    if tracker_type == "OCSORT":
        try:
            from ocsort.ocsort import OCSort

            if tracker_config_path is None:
                config_path = DEFAULT_OCSORT_CONFIG_PATH
            else:
                config_path = tracker_config_path

            config = get_config(config_path)
            oc_sort = OCSort(
                det_thresh=conf_th,
                max_age=config.OC_SORT.MAX_AGE,
                min_hits=config.OC_SORT.MIN_HITS,
                iou_threshold=iou_th,
                delta_t=config.OC_SORT.DELTA_T,
                asso_func=config.OC_SORT.ASSO_FUNC,
                inertia=config.OC_SORT.INERTIA,
                use_byte=config.OC_SORT.USE_BYTE,
            )
            return oc_sort

        except ImportError:
            raise ImportError("Please install ocsort: pip install ocsort")

    elif tracker_type == "BYTETRACK":
        try:
            from bytetracker.byte_tracker import BYTETracker

            if tracker_config_path is None:
                config_path = DEFAULT_BYTETRACK_CONFIG_PATH
            else:
                config_path = tracker_config_path

            config = get_config(config_path)

            byte_tracker = BYTETracker(
                track_thresh=conf_th,
                track_buffer=config.BYTE_TRACK.TRACK_BUFFER,
                frame_rate=config.BYTE_TRACK.FRAME_RATE,
            )
            return byte_tracker

        except ImportError:
            raise ImportError("Please install bytetracker: pip install bytetracker")

    elif tracker_type == "NORFAIR":
        try:
            from norfair_tracker.norfair import NorFairTracker

            if tracker_config_path is None:
                config_path = DEFAULT_NORFAIR_CONFIG_PATH
            else:
                config_path = tracker_config_path

            config = get_config(config_path)
            norfair_tracker = NorFairTracker(
                distance_function=config.NORFAIR_TRACK.DISTANCE_FUNCTION,
                distance_threshold=config.NORFAIR_TRACK.DISTANCE_THRESHOLD,
                hit_counter_max=config.NORFAIR_TRACK.HIT_COUNTER_MAX,
                initialization_delay=config.NORFAIR_TRACK.INITIALIZATION_DELAY,
                pointwise_hit_counter_max=config.NORFAIR_TRACK.POINTWISE_HIT_COUNTER_MAX,
                detection_threshold=config.NORFAIR_TRACK.DETECTION_THRESHOLD,
                past_detections_length=config.NORFAIR_TRACK.PAST_DETECTIONS_LENGTH,
                reid_distance_threshold=config.NORFAIR_TRACK.REID_DISTANCE_THRESHOLD,
                reid_hit_counter_max=config.NORFAIR_TRACK.REID_HIT_COUNTER_MAX,
            )
            return norfair_tracker
        except ImportError:
            raise ImportError("Please install norfair: pip install norfair-tracker")

    elif tracker_type == "SORT":
        try:
            from sort.tracker import SortTracker

            if tracker_config_path is None:
                config_path = DEFAULT_SORT_CONFIG_PATH
            else:
                config_path = tracker_config_path

            config = get_config(config_path)
            sort_tracker = SortTracker(
                max_age=config.SORT_TRACK.MAX_AGE,
                min_hits=config.SORT_TRACK.MIN_HITS,
                iou_threshold=iou_th,
            )
            return sort_tracker

        except ImportError:
            raise ImportError("Please install sort: pip install sort-track")

    elif tracker_type == "STRONGSORT":
        try:
            from strongsort.strong_sort import StrongSORT

            if tracker_config_path is None:
                config_path = DEFAULT_STRONGSORT_CONFIG_PATH
            else:
                config_path = tracker_config_path

            config = get_config(config_path)
            strong_sort = StrongSORT(
                tracker_weight_path,
                device,
                half,
                max_dist=config.STRONG_SORT.MAX_DIST,
                max_iou_distance=config.STRONG_SORT.MAX_IOU_DISTANCE,
                max_age=config.STRONG_SORT.MAX_AGE,
                n_init=config.STRONG_SORT.N_INIT,
                nn_budget=config.STRONG_SORT.NN_BUDGET,
                mc_lambda=config.STRONG_SORT.MC_LAMBDA,
                ema_alpha=config.STRONG_SORT.EMA_ALPHA,
            )
            return strong_sort

        except ImportError:
            raise ImportError("Please install strongsort: pip install strongsort")


def load_tracker(
    config_path: str = None,
    tracker_type: str = None,
    tracker_weight_path: str = "osnet_x1_0_msmt17.pt",
    tracker_config_path: str = None,
):
    config = get_config(config_path)
    tracker_module = create_tracker(
        tracker_type=tracker_type,
        tracker_weight_path=Path(tracker_weight_path),
        tracker_config_path=tracker_config_path,
        device=config.DETECTOR_CONFIG.DEVICE,
        half=config.DETECTOR_CONFIG.HALF,
        conf_th=config.DETECTOR_CONFIG.CONF_TH,
        iou_th=config.DETECTOR_CONFIG.IOU_TH,
    )
    return tracker_module



import cv2
import torch
from tqdm import tqdm

from torchyolo.tracker.tracker_zoo import load_tracker
from torchyolo.utils.config_utils import get_config
from torchyolo.utils.dataset import LoadData, create_video_writer
from torchyolo.utils.download import attempt_download_from_hub
from torchyolo.utils.object_vis import video_vis


class Yolov8DetectionModel:
    def __init__(
        self,
        config_path: str,
        model_path: str = "yolov8s.pt",
    ):

        self.model_path = model_path
        self.load_config(config_path)
        self.load_model()

    def load_config(self, config_path: str):
        self.config_path = config_path
        config = get_config(config_path)
        self.output_path = config.DATA_CONFIG.OUTPUT_PATH
        self.device = config.DETECTOR_CONFIG.DEVICE
        self.conf = config.DETECTOR_CONFIG.CONF_TH
        self.iou = config.DETECTOR_CONFIG.IOU_TH
        self.image_size = config.DETECTOR_CONFIG.IMAGE_SIZE
        self.save = config.DATA_CONFIG.SAVE
        self.show = config.DATA_CONFIG.SHOW
        self.hf_model = config.DETECTOR_CONFIG.HUGGING_FACE_MODEL

    def load_model(self):
        try:
            from ultralytics import YOLO

            if self.hf_model:
                hf_model_path = attempt_download_from_hub(self.model_path)
                model = YOLO(hf_model_path)
                model.conf = self.conf
                model.iou = self.iou
                self.model = model

            else:
                model = YOLO(self.model_path)
                model.conf = self.conf
                model.iou = self.iou
                self.model = model

        except ImportError:
            raise ImportError('Please run "pip install ultralytics" ' "to install YOLOv8 first for YOLOv8 inference.")

    def predict(
        self,
        source: str = None,
        tracker_type: str = None,
        tracker_weight_path: str = None,
        tracker_config_path: str = None,
    ):
        if tracker_type == "STRONGSORT":
            tracker_module = load_tracker(
                config_path=self.config_path,
                tracker_type=tracker_type,
                tracker_weight_path=tracker_weight_path,
                tracker_config_path=tracker_config_path,
            )

        else:
            tracker_module = load_tracker(
                config_path=self.config_path,
                tracker_type=tracker_type,
                tracker_config_path=tracker_config_path,
            )

        tracker_outputs = [None]
        dataset = LoadData(source)
        video_writer = create_video_writer(video_path=source, output_path=self.output_path)

        for img_src, img_path, vid_cap in tqdm(dataset):
            results = self.model.predict(img_src, imgsz=self.image_size)
            if tracker_type is not None:
                for image_id, prediction in enumerate(results):
                    boxes = prediction[:].boxes.xyxy
                    score = prediction[:].boxes.conf
                    category_id = prediction[:].boxes.cls
                    dets = torch.cat((boxes, score.unsqueeze(1), category_id.unsqueeze(1)), dim=1)
                    tracker_outputs[image_id] = tracker_module.update(dets.cpu(), img_src)
                    for output in tracker_outputs[image_id]:
                        bbox, track_id, category_id, score = (
                            output[:4],
                            int(output[4]),
                            output[5],
                            output[6],
                        )
                        category_name = self.model.model.names[int(category_id)]
                        label = f"Id:{track_id} {category_name} {float(score):.2f}"

                        if self.save or self.show:
                            img_src = video_vis(
                                bbox=bbox,
                                label=label,
                                frame=img_src,
                                object_id=int(category_id),
                            )
                if self.save:
                    video_writer.write(img_src)

            else:

                for image_id, prediction in enumerate(results[0].boxes.cpu().numpy()):

                    bbox, category_id, score = (
                        prediction.xyxy,
                        prediction.cls,
                        prediction.conf,
                    )
                    category_name = self.model.model.names[int(category_id)]
                    label = f"{category_name} {float(score):.2f}"

                    if self.save or self.show:
                        frame = video_vis(
                            bbox=bbox[0],
                            label=label,
                            frame=img_src,
                            object_id=int(category_id),
                        )
                    if self.save:
                        if source.endswith(".mp4"):
                            video_writer.write(frame)
                        else:
                            cv2.imwrite("output.jpg", frame)



import cv2
import numpy as np


class Colors:
    # Ultralytics color palette https://ultralytics.com/
    def __init__(self):
        # hex = matplotlib.colors.TABLEAU_COLORS.values()
        hexs = (
            "FF3838",
            "FF9D97",
            "FF701F",
            "FFB21D",
            "CFD231",
            "48F90A",
            "92CC17",
            "3DDB86",
            "1A9334",
            "00D4BB",
            "2C99A8",
            "00C2FF",
            "344593",
            "6473FF",
            "0018EC",
            "8438FF",
            "520085",
            "CB38FF",
            "FF95C8",
            "FF37C7",
        )
        self.palette = [self.hex2rgb(f"#{c}") for c in hexs]
        self.n = len(self.palette)

    def __call__(self, i, bgr=False):
        c = self.palette[int(i) % self.n]
        return (c[2], c[1], c[0]) if bgr else c

    @staticmethod
    def hex2rgb(h):  # rgb order (PIL)
        return tuple(int(h[1 + i : 1 + i + 2], 16) for i in (0, 2, 4))


def video_vis(
    object_id,
    label,
    frame,
    bbox,
) -> np.ndarray:
    x, y, w, h = int(bbox[0]), int(bbox[1]), int(bbox[2]), int(bbox[3])
    MIN_FONT_SCALE = 0.7
    color = Colors()(object_id % 10)
    txt_color = (0, 0, 0) if np.mean(color) > 0.5 else (255, 255, 255)
    font_scale = max(MIN_FONT_SCALE, 0.3 * (w + h) / 600)
    thickness = 2
    txt_size = cv2.getTextSize(label, cv2.FONT_HERSHEY_SIMPLEX, font_scale, thickness)[0]
    cv2.rectangle(frame, (x, y), (w, h), color, thickness)  # object box
    cv2.rectangle(frame, (x, y - txt_size[1]), (x + txt_size[0], y), color, -1)  # object label box
    cv2.putText(
        frame,
        label,
        (x, y - 2),
        cv2.FONT_HERSHEY_SIMPLEX,
        font_scale,
        txt_color,
        thickness,
        cv2.LINE_AA,
    )
    return frame



#!/usr/bin/env python3
# -*- coding:utf-8 -*-
# Copyright (c) Megvii, Inc. and its affiliates.

COCO_CLASSES = (
    "person",
    "bicycle",
    "car",
    "motorcycle",
    "airplane",
    "bus",
    "train",
    "truck",
    "boat",
    "traffic light",
    "fire hydrant",
    "stop sign",
    "parking meter",
    "bench",
    "bird",
    "cat",
    "dog",
    "horse",
    "sheep",
    "cow",
    "elephant",
    "bear",
    "zebra",
    "giraffe",
    "backpack",
    "umbrella",
    "handbag",
    "tie",
    "suitcase",
    "frisbee",
    "skis",
    "snowboard",
    "sports ball",
    "kite",
    "baseball bat",
    "baseball glove",
    "skateboard",
    "surfboard",
    "tennis racket",
    "bottle",
    "wine glass",
    "cup",
    "fork",
    "knife",
    "spoon",
    "bowl",
    "banana",
    "apple",
    "sandwich",
    "orange",
    "broccoli",
    "carrot",
    "hot dog",
    "pizza",
    "donut",
    "cake",
    "chair",
    "couch",
    "potted plant",
    "bed",
    "dining table",
    "toilet",
    "tv",
    "laptop",
    "mouse",
    "remote",
    "keyboard",
    "cell phone",
    "microwave",
    "oven",
    "toaster",
    "sink",
    "refrigerator",
    "book",
    "clock",
    "vase",
    "scissors",
    "teddy bear",
    "hair drier",
    "toothbrush",
)



import cv2
from tqdm import tqdm

from torchyolo.tracker.tracker_zoo import load_tracker
from torchyolo.utils.config_utils import get_config
from torchyolo.utils.dataset import LoadData, create_video_writer
from torchyolo.utils.object_vis import video_vis


class Yolov5DetectionModel:
    def __init__(
        self,
        config_path: str,
        model_path: str = "yolov5n.pt",
    ):

        self.model_path = model_path
        self.load_config(config_path)
        self.load_model()

    def load_config(self, config_path: str):
        self.config_path = config_path
        config = get_config(config_path)
        self.output_path = config.DATA_CONFIG.OUTPUT_PATH
        self.conf = config.DETECTOR_CONFIG.CONF_TH
        self.iou = config.DETECTOR_CONFIG.IOU_TH
        self.image_size = config.DETECTOR_CONFIG.IMAGE_SIZE
        self.device = config.DETECTOR_CONFIG.DEVICE
        self.save = config.DATA_CONFIG.SAVE
        self.show = config.DATA_CONFIG.SHOW

    def load_model(self):
        try:
            import yolov5

            model = yolov5.load(self.model_path, device=self.device)
            model.conf = self.conf
            model.iou = self.iou
            self.model = model

        except ImportError:
            raise ImportError('Please run "pip install -U yolov5" ' "to install YOLOv5 first for YOLOv5 inference.")

    def predict(
        self,
        source: str = None,
        tracker_type: str = None,
        tracker_weight_path: str = None,
        tracker_config_path: str = None,
    ):
        if tracker_type == "STRONGSORT":
            tracker_module = load_tracker(
                config_path=self.config_path,
                tracker_type=tracker_type,
                tracker_weight_path=tracker_weight_path,
                tracker_config_path=tracker_config_path,
            )

        else:
            tracker_module = load_tracker(
                config_path=self.config_path,
                tracker_type=tracker_type,
                tracker_config_path=tracker_config_path,
            )

        tracker_outputs = [None]
        dataset = LoadData(source)
        video_writer = create_video_writer(video_path=source, output_path=self.output_path)
        for img_src, img_path, vid_cap in tqdm(dataset):
            results = self.model(img_src, size=self.image_size)
            for image_id, prediction in enumerate(results.pred):
                if tracker_type is not None:
                    tracker_outputs[image_id] = tracker_module.update(prediction.cpu(), img_src)
                    for output in tracker_outputs[image_id]:
                        bbox, track_id, category_id, score = (
                            output[:4],
                            int(output[4]),
                            output[5],
                            output[6],
                        )
                        category_name = self.model.names[int(category_id)]
                        label = f"Id:{track_id} {category_name} {float(score):.2f}"

                        if self.save or self.show:
                            img_src = video_vis(
                                bbox=bbox,
                                label=label,
                                frame=img_src,
                                object_id=int(category_id),
                            )
                    if self.save:
                        video_writer.write(img_src)

                else:
                    for pred in prediction.cpu().detach().numpy():
                        x1, y1, x2, y2 = (
                            int(pred[0]),
                            int(pred[1]),
                            int(pred[2]),
                            int(pred[3]),
                        )
                        bbox = [x1, y1, x2, y2]
                        score = pred[4]
                        category_name = self.model.names[int(pred[5])]
                        category_id = int(pred[5])
                        label = f"{category_name} {score:.2f}"

                    frame = video_vis(
                        bbox=bbox,
                        label=label,
                        frame=img_src,
                        object_id=category_id,
                    )
                    if self.save:
                        if source.endswith(".mp4"):
                            video_writer.write(frame)
                        else:
                            cv2.imwrite("output.jpg", frame)



import glob
import os
from pathlib import Path

import cv2

from torchyolo.utils.file_utils import create_dir

IMG_FORMATS = "bmp", "dng", "jpeg", "jpg", "mpo", "png", "tif", "tiff", "webp", "pfm"  # include image suffixes
VID_FORMATS = "asf", "avi", "gif", "m4v", "mkv", "mov", "mp4", "mpeg", "mpg", "ts", "wmv"  # include video suffixes


class LoadData:
    def __init__(self, path):
        p = str(Path(path).resolve())  # os-agnostic absolute path
        if os.path.isdir(p):
            files = sorted(glob.glob(os.path.join(p, "**/*.*"), recursive=True))  # dir
        elif os.path.isfile(p):
            files = [p]  # files
        else:
            raise FileNotFoundError(f"Invalid path {p}")
        imgp = [i for i in files if i.split(".")[-1] in IMG_FORMATS]
        vidp = [v for v in files if v.split(".")[-1] in VID_FORMATS]
        self.files = imgp + vidp
        self.nf = len(self.files)
        self.type = "image"
        if any(vidp):
            self.add_video(vidp[0])  # new video
        else:
            self.cap = None

    @staticmethod
    def checkext(path):
        file_type = "image" if path.split(".")[-1].lower() in IMG_FORMATS else "video"
        return file_type

    def __iter__(self):
        self.count = 0
        return self

    def __next__(self):
        if self.count == self.nf:
            raise StopIteration
        path = self.files[self.count]
        if self.checkext(path) == "video":
            self.type = "video"
            ret_val, img = self.cap.read()
            while not ret_val:
                self.count += 1
                self.cap.release()
                if self.count == self.nf:  # last video
                    raise StopIteration
                path = self.files[self.count]
                self.add_video(path)
                ret_val, img = self.cap.read()
        else:
            # Read image
            self.count += 1
            img = cv2.imread(path)  # BGR
        return img, path, self.cap

    def add_video(self, path):
        self.frame = 0
        self.cap = cv2.VideoCapture(path)
        self.frames = int(self.cap.get(cv2.CAP_PROP_FRAME_COUNT))

    def __len__(self):
        return self.nf  # number of files


def create_video_writer(video_path, output_path, fps=None) -> cv2.VideoWriter:
    """
    This function is used to create video writer.
    Args:
        video_path: video path
        output_path: output path
        fps: fps
    Returns:
        video writer
    """
    if fps is None:
        cap = cv2.VideoCapture(video_path)
        fps = cap.get(cv2.CAP_PROP_FPS)

    cap = cv2.VideoCapture(video_path)
    width = int(cap.get(cv2.CAP_PROP_FRAME_WIDTH))
    height = int(cap.get(cv2.CAP_PROP_FRAME_HEIGHT))
    size = (width, height)
    fourcc = cv2.VideoWriter_fourcc(*"mp4v")
    videoWriter = cv2.VideoWriter(output_path, fourcc, fps, size)
    return videoWriter


def get_video_formats():
    """
    This function is used to get video formats.
    Returns:
        video formats
    """
    extensions = [".asf", ".avi", ".gif", ".m4v", ".mkv", ".mov", ".mp4", ".mpeg", ".mpg", ".ts", ".wmv"]
    video_formats = [x.upper() for x in extensions] + extensions
    return video_formats
